precision mediump float;

varying mediump vec2 textureCoordinate;

uniform sampler2D inputImageTexture;
uniform vec2 singleStepOffset;
uniform mediump float params;

const highp vec3 W = vec3(0.299,0.587,0.114);
const highp mat3 saturateMatrix = mat3(
     1.1102, -0.0598, -0.061,
     -0.0774, 1.0826, -0.1186,
     -0.0228, -0.0228, 1.1772);
const vec4 paramsConst = vec4(1.0 - 0.6 * 0.7,
                    1.0 - 0.3 * 0.7,
                    0.1 + 0.3 * 0.5,
                    0.1 + 0.3 * 0.5);
vec2 blurCoordinates[24];

float hardLight(float color)
{
	if(color <= 0.5)
		color = color * color * 2.0;
	else
		color = 1.0 - ((1.0 - color)*(1.0 - color) * 2.0);
	return color;
}

void main(){

    vec3 centralColor = texture2D(inputImageTexture, textureCoordinate).rgb;
    blurCoordinates[4] = textureCoordinate.xy + singleStepOffset * vec2(0.0, -8.0);
    blurCoordinates[5] = textureCoordinate.xy + singleStepOffset * vec2(0.0, 8.0);
    blurCoordinates[6] = textureCoordinate.xy + singleStepOffset * vec2(-8.0, 0.0);
    blurCoordinates[7] = textureCoordinate.xy + singleStepOffset * vec2(8.0, 0.0);

    highp float sampleColor = centralColor.g * 8.0;
    sampleColor += texture2D(inputImageTexture, blurCoordinates[0]).g;
    sampleColor += texture2D(inputImageTexture, blurCoordinates[1]).g;
    sampleColor += texture2D(inputImageTexture, blurCoordinates[2]).g;
    sampleColor += texture2D(inputImageTexture, blurCoordinates[3]).g;

    sampleColor = sampleColor / 12.0;

     highp float highPass = centralColor.g - sampleColor + 0.5;

    for(int i = 0; i < 5;i++)
    {
        highPass = hardLight(highPass);
    }
    float luminance = dot(centralColor, W);

    float alpha = pow(luminance, paramsConst.r);

    vec3 smoothColor = centralColor + (centralColor-vec3(highPass))*alpha*0.1;

    smoothColor.r = clamp(pow(smoothColor.r, paramsConst.g), 0.0, 1.0);
    smoothColor.g = clamp(pow(smoothColor.g, paramsConst.g), 0.0, 1.0);
    smoothColor.b = clamp(pow(smoothColor.b, paramsConst.g), 0.0, 1.0);

    highp vec3 lvse = vec3(1.0)-(vec3(1.0)-smoothColor)*(vec3(1.0)-centralColor);
    highp vec3 bianliang = max(smoothColor, centralColor);
    highp vec3 rouguang = 2.0 * centralColor * smoothColor + centralColor * centralColor- 2.0 * centralColor * centralColor * smoothColor;

    highp vec4 tmp = vec4(mix(centralColor, lvse, alpha), 1.0);
    tmp.rgb = mix(tmp.rgb, bianliang, alpha);
    tmp.rgb = mix(tmp.rgb, rouguang, paramsConst.b);

    highp vec3 satcolor = tmp.rgb * saturateMatrix;
    tmp.rgb = mix(tmp.rgb, satcolor, paramsConst.a);
    gl_FragColor = tmp;
}