#extension GL_OES_EGL_image_external : require
precision mediump float;
varying vec2 textureCoordinate;
uniform samplerExternalOES s_texture;
uniform sampler2D maskTexture;
uniform highp vec2 singleStepOffset;
uniform highp float brightness;
uniform highp vec4 params;
const highp vec3 W = vec3(0.299, 0.587, 0.114);
const highp mat3 saturateMatrix = mat3(
     1.1102, -0.0598, -0.061,
     -0.0774, 1.0826, -0.1186,
-0.0228, -0.0228, 1.1772);
highp vec2 blurCoordinates[2];

highp float hardLight(highp float color) {
     if (color <= 0.5)
         color = color * color * 2.0;
     else
         color = 1.0 - ((1.0 - color)*(1.0 - color) * 2.0);
     return color;
}

mediump float myrand(highp vec2 co){
    return fract(sin(dot(co.xy ,vec2(12.91,78.23))) * 12.11);
}

void main() {
     highp vec3 centralColor = texture2D(s_texture, textureCoordinate).rgb;

     float ratio = myrand(textureCoordinate);
     float randx = ratio * 12.0;
     float randy = 12.0 - randx;
     blurCoordinates[0] = textureCoordinate.xy + singleStepOffset * vec2(randx, randy);
     blurCoordinates[1] = textureCoordinate.xy + singleStepOffset * vec2(-randx, -randy);
     highp float sampleColor = centralColor.g * 4.0;
     sampleColor += texture2D(s_texture, blurCoordinates[0]).g;
     sampleColor += texture2D(s_texture, blurCoordinates[1]).g;
     sampleColor = sampleColor / 6.0;

     highp float highPass = centralColor.g - sampleColor + 0.5;

     for (int i = 0; i < 5; i++) {
         highPass = hardLight(highPass);
     }
     highp float lumance = dot(centralColor, W);

     highp float alpha = pow(lumance, params.r);

     highp vec3 smoothColor = centralColor + (centralColor-vec3(highPass))*alpha*0.1;

     smoothColor.r = clamp(pow(smoothColor.r, params.g), 0.0, 1.0);
     smoothColor.g = clamp(pow(smoothColor.g, params.g), 0.0, 1.0);
     smoothColor.b = clamp(pow(smoothColor.b, params.g), 0.0, 1.0);

     highp vec3 lvse = vec3(1.0)-(vec3(1.0)-smoothColor)*(vec3(1.0)-centralColor);
     highp vec3 bianliang = max(smoothColor, centralColor);
     highp vec3 rouguang = 2.0*centralColor*smoothColor + centralColor*centralColor- 2.0*centralColor*centralColor*smoothColor;

     gl_FragColor = vec4(mix(centralColor, lvse, alpha), 1.0);
     gl_FragColor.rgb = mix(gl_FragColor.rgb, bianliang, alpha);
     gl_FragColor.rgb = mix(gl_FragColor.rgb, rouguang, params.b);

     highp vec3 satcolor = gl_FragColor.rgb * saturateMatrix;
     gl_FragColor.rgb = mix(gl_FragColor.rgb, satcolor, params.a);
     gl_FragColor.rgb = vec3(gl_FragColor.rgb + vec3(brightness));
}